<?php

declare(strict_types = 1);

namespace Drupal\group_finder\Context;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Plugin\Context\Context;
use Drupal\Core\Plugin\Context\ContextProviderInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Sets the group as a context if the group finder finds a group.
 */
class GroupFinderContext implements ContextProviderInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getRuntimeContexts(array $unqualified_context_ids): array {
    // Create an optional context definition for group entities.
    $context_definition = EntityContextDefinition::fromEntityTypeId('group')
      ->setRequired(FALSE);

    // If we use dependency injection in the context provider for the group
    // finder, it would be initialized too early, when the route is not ready
    // for the group finder plugins to detect the group.
    /** @var \Drupal\group_finder\GroupFinderFactoryInterface $group_finder */
    $group_finder = \Drupal::service('group_finder');

    // Cache this context per group on the route.
    $cacheability = new CacheableMetadata();

    // We need to use the general 'route' cache context. Using 'route.group' is
    // not enough for this context provider, as the group provider service
    // finds groups on routes that are not group routes (nodes, views).
    $cacheability->setCacheContexts(['route']);

    // Create a context from the definition and the group retrieved from group
    // finder.
    $context = new Context($context_definition, $group_finder->get()?->getGroup());
    $context->addCacheableDependency($cacheability);

    return ['group' => $context];
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailableContexts(): array {
    return ['group' => EntityContext::fromEntityTypeId('group', $this->t('Group from group finder.'))];
  }

}
