<?php

declare(strict_types = 1);

namespace Drupal\group_finder;

/**
 * Class GroupFinderFactory.
 *
 * A simple factory class used for getting the applicable group finder plugin.
 */
class GroupFinderFactory implements GroupFinderFactoryInterface {

  /**
   * The group finder plugin manager.
   *
   * @var \Drupal\group_finder\GroupFinderManager
   */
  protected $groupFinder;

  /**
   * The applicable group finder plugin.
   *
   * @var \Drupal\group_finder\GroupFinderInterface
   */
  protected $plugin;

  /**
   * GroupFinderFactory constructor.
   *
   * @param \Drupal\group_finder\GroupFinderManager $group_finder_manager
   *   Group finder plugin manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function __construct(GroupFinderManager $group_finder_manager) {
    $this->groupFinder = $group_finder_manager;
    $this->find();
  }

  /**
   * find the applicable group finder plugin.
   *
   * Processing group finders one by one until we find an active one.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  private function find(): void {
    $plugins = $this->groupFinder->getDefinitions();

    foreach ($plugins as $plugin_id => $definition) {
      /** @var \Drupal\group_finder\GroupFinderInterface $pluginInstance */
      $pluginInstance = $this->groupFinder->createInstance($plugin_id);
      if ($pluginInstance && $pluginInstance->isApplicable()) {
        $this->plugin = $pluginInstance;
        break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function get(): ?GroupFinderInterface {
    return $this->plugin ?? NULL;
  }

}
